--[[
  Based on an idea of:
  
  Seb Lee-Delisle
  Blog : www.sebleedelisle.com
  Company : www.pluginmedia.net
   
  This work is licensed under a Creative Commons  2.0 License.
 
  Full details at 
  http:--creativecommons.org/licenses/by/2.0/uk/
  
  Modified by Bill Rigas
  sort.coder@gmail.com
]]

-- Base class
particleClass = gideros.class(Sprite)

-- Derived classes
ccSnowParticle = gideros.class(particleClass)
ccExplosionParticle = gideros.class(particleClass)

function ccSnowParticle:init(ptable, start)
  self.particleTable = ptable
  
  self.totalParticles = self.particleTable["maxParticles"]
  self.duration = ptable["duration"]
  
  self.emissionRate = self.totalParticles / ptable["particleLifespan"]
  self.imgTexture = Texture.new("fire.png")
  
  if start ~= nil then
    if not start then self:stop() end
  end
end

function ccExplosionParticle:init(ptable, start)
  self.particleTable = ptable
  
  self.totalParticles = self.particleTable["maxParticles"]
  self.duration = ptable["duration"]
  
  self.emissionRate = self.totalParticles / ptable["duration"]
  self.imgTexture = Texture.new("fire.png")
  
  if start ~= nil then
    if not start then self:stop() end
  end
end

function particleClass:init()
  self.totalParticles = 250		-- number of particles
 
  self.particles = {}	-- particle table
  self.spare = {}		-- spare particle table

  self.emissionRate = 0
  self.imgTexture = nil
  self.emitCounter = 0
  self.elapsed = 0
  self.rate = 0
  self.startParticle = true
  self.particleTable = nil
  self.duration = -1
  
  self:addEventListener(Event.ENTER_FRAME, self.enterFrameHandler, self)
end

function particleClass:start()
  self.startParticle = true
end

function particleClass:stop()
  self.startParticle = false
end

function particleClass:isStopped()
  return not self.startParticle
end

function particleClass:enterFrameHandler(event)
	local dt = event.deltaTime
	
	if self.startParticle then
	  self:updateParticles(dt)
    end
end

function particleClass:restart()
  self.emitCounter = 0
  self.elapsed = 0
  
  -- from spare to particles table
  while #self.spare > 0 do
    local particle = table.remove(self.spare, 1)
	table.insert(self.particles, particle)
	particle:reset()
  end
end
 
-- update existing particles
function particleClass:updateParticles(dt)
	self.rate = 1 / self.emissionRate
	self.emitCounter = self.emitCounter + dt
	
	local appWidth = application:getContentWidth()
	
	while (#self.particles < self.totalParticles) and (self.emitCounter > self.rate) do
		local spark = self:createParticle(math.random(0, appWidth), math.random(0, 40))
		table.insert(self.particles, spark)
		self.emitCounter = self.emitCounter - self.rate
	end
	
	self.elapsed = self.elapsed + dt

	local particle

	-- go through the array of particles...
	local i = 1
	local len = #self.particles
	local flag = len > 1
 
	while flag do
		-- ... and update each one
		particle = self.particles[i]
		
		if not particle.disabled then
			particle:update(dt)
			--if particle.sprite:getWidth() < 2 then
			if particle.timeToLive < 0 then
			   if (particle.duration == -1) and (particle.duration < self.elapsed) then
				 particle:reset()
			   end
				particle:disable()
				table.insert(self.spare, table.remove(self.particles, i))
				-- table size has changed, reflect that in variables
				len = len - 1
				i = i - 1
			end
		end
 
		i = i + 1
		-- check whether we reached end of the table
		if i > len then 
			flag = false 
		end
	end
	
	if (#self.particles == 0) and (self.duration >= 0) then
	  self:stop()
	end
end
 
-- draw spark shape, or use image as a particle
function particleClass:createParticle(x, y)
	local particle
	
	if self.particleTable == nil then return end
 
	if #self.spare == 0 then
		-- or use image
		local shape = Bitmap.new(self.imgTexture)
		self:addChild(shape)
		shape:setPosition(x, y);
		particle = cParticle.new(shape, self.particleTable)
	else 
		-- reuse particle
		particle = table.remove(self.spare, 1)
		particle.sprite:setPosition(x, y)
		particle:enable()
	end
 
	return particle
end
